#ifndef __SCENEENTITY_H
#define __SCENEENTITY_H

#include "glInterface.h"
#include <Cg/cg.h>
#include <Cg/cgGL.h>
#include "common.h"
#include "AxisAlignedBox3.h"
#include "Triangle3.h"
#include "LODInfo.h"


namespace CHCDemoEngine 
{

class Material;
class Geometry;
class RenderState;
class Transform3;


/** Class representing a scene entity.
    A scene entity basically consists of geometry, transformation, and a material
*/
class SceneEntity
{
	friend class RenderQueue;

public:
	
	/** Creates a scene entity.
	*/
	SceneEntity(Transform3 *trafo);

	~SceneEntity();
	/** Renders this node.
	*/
	void Render(RenderState *state);	
	/** Set pointer to the shape
	*/
	void AddShape(Shape *shape);
	/** See set
	*/
	inline Shape *GetShape(int i) const { return mShapes[i]; }
	/** Returns number of shapes in vector.
	*/
	inline int GetNumShapes() { return (int)mShapes.size(); }
	/** Set pointer to the geometry
	*/
	void SetTransform(Transform3 *trafo);
	/** Returns the transformed bounding box.
	*/
	AxisAlignedBox3 GetBoundingBox() const;
	/** set frame where we last rendered this node
	*/
	void SetLastRendered(int lastRendered);
	/** returns frame where we last visited this node
	*/
	int GetLastRendered() const;
	/** Returns the trafo of this scene entity.
	*/
	inline Transform3 *GetTransform() const  { return mTransform; }


	void GetCurrentLODLevel(ShapeContainer &shapes);

protected:


	/// transform matrix 
	Transform3 *mTransform;
	/// Stores information about the LOD levels
	LODInfoContainer mLODInfos;
	/// the renderable shapes
	ShapeContainer mShapes;
	/// when this entity was last rendered
	int mLastRendered;
};

}

#endif // __SCENEENTITY_H